@@ -23,7 +23,7 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/font/fontfile/fontdir.c,v 3.24 2006/01/09 14:59:34 dawes Exp $ */
+/* $XFree86: xc/lib/font/fontfile/fontdir.c,v 3.25 2007/04/05 16:08:59 tsi Exp $ */
 
 /*
  * Author:  Keith Packard, MIT X Consortium
@@ -37,6 +37,9 @@ FontFileInitTable (FontTablePtr table, int size)
 {
     if (size)
     {
+	if ((size < 0) || (size > (0x7fffffff / sizeof(FontEntryRec))))
+	    return FALSE;
+
 	table->entries = (FontEntryPtr) xalloc(sizeof(FontEntryRec) * size);
 	if (!table->entries)
 	    return FALSE;
