@@ -1,5 +1,3 @@
-/* Header:   //Mercury/Projects/archives/XFree86/4.0/smi_accel.c-arc   1.16   03 Jan 2001 13:29:06   Frido  $ */
-
 /*
 Copyright (C) 1994-1999 The XFree86 Project, Inc.  All Rights Reserved.
 Copyright (C) 2000 Silicon Motion, Inc.  All Rights Reserved.
@@ -26,7 +24,7 @@ Silicon Motion shall not be used in advertising or otherwise to promote the
 sale, use or other dealings in this Software without prior written
 authorization from the XFree86 Project and silicon Motion.
 */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/siliconmotion/smi_accel.c,v 1.9 2003/10/08 11:13:01 eich Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/siliconmotion/smi_accel.c,v 1.13 2006/03/09 17:37:08 tsi Exp $ */
 
 #include "smi.h"
 
@@ -118,7 +116,11 @@ SMI_AccelInit(ScreenPtr pScreen)
 	/* Color Expansion Fills */
 	infoPtr->CPUToScreenColorExpandFillFlags = ROP_NEEDS_SOURCE
 											 | NO_PLANEMASK
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+											 | BIT_ORDER_IN_BYTE_LSBFIRST
+#else
 											 | BIT_ORDER_IN_BYTE_MSBFIRST
+#endif
 											 | LEFT_EDGE_CLIPPING
 											 | CPU_TRANSFER_PAD_DWORD
 											 | SCANLINE_PAD_DWORD;
@@ -241,7 +243,7 @@ void
 SMI_GEReset(ScrnInfoPtr pScrn, int from_timeout, int line, char *file)
 {
 	SMIPtr pSmi = SMIPTR(pScrn);
-	CARD8 tmp;
+	CARD8 tmp = 0;
 
 	ENTER_PROC("SMI_GEReset");
 
@@ -257,12 +259,25 @@ SMI_GEReset(ScrnInfoPtr pScrn, int from_timeout, int line, char *file)
 		WaitIdleEmpty();
 	}
 
-	tmp = VGAIN8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15);
-	VGAOUT8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15, tmp | 0x30);
+	if (pSmi->Chipset == SMI_MSOC)
+	{
+		CARD32 itmp = READ_SCR(pSmi, SCR00) & ~0x3000;
+		WRITE_SCR(pSmi, SCR00, (itmp | 0x3000));
+		WRITE_SCR(pSmi, SCR00, itmp);
+	}
+	else
+	{
+		tmp = VGAIN8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15);
+		VGAOUT8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15, tmp | 0x30);
+	}
 
 	WaitIdleEmpty();
 
-	VGAOUT8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15, tmp);
+	if (pSmi->Chipset != SMI_MSOC)
+	{
+		VGAOUT8_INDEX(pSmi, VGA_SEQ_INDEX, VGA_SEQ_DATA, 0x15, tmp);
+	}
+
 	SMI_EngineReset(pScrn);
 
 	LEAVE_PROC("SMI_GEReset");
@@ -475,6 +490,9 @@ SMI_SubsequentSolidFillRect(ScrnInfoPtr pScrn, int x, int y, int w, int h)
 	ENTER_PROC("SMI_SubsequentSolidFillRect");
 	DEBUG((VERBLEV, "x=%d y=%d w=%d h=%d\n", x, y, w, h));
 
+	if (x < 0) x = 0;
+	if (y < 0) y = 0;
+
 	if (pScrn->bitsPerPixel == 24)
 	{
 		x *= 3;
@@ -860,7 +878,7 @@ SMI_SubsequentImageWriteRect(ScrnInfoPtr pScrn, int x, int y, int w, int h,
 		}
 	}
 	WRITE_DPR(pSmi, 0x00, 0);
-	WRITE_DPR(pSmi, 0x04, (x << 16) | (y * 0xFFFF));
+	WRITE_DPR(pSmi, 0x04, (x << 16) | (y & 0xFFFF));
 	WRITE_DPR(pSmi, 0x08, (w << 16) | (h & 0xFFFF));
 	WRITE_DPR(pSmi, 0x0C, pSmi->AccelCmd);
 
@@ -901,6 +919,12 @@ SMI_SetClippingRectangle(ScrnInfoPtr pScrn, int left, int top, int right,
 		}
 	}
 
+	if (pSmi->Chipset == SMI_MSOC)
+	{
+		bottom++;
+		right++;
+	}
+
 	pSmi->ScissorsLeft = (top << 16) | (left & 0xFFFF) | 0x2000;
 	pSmi->ScissorsRight = (bottom << 16) | (right & 0xFFFF);
 
