@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/xf86Pci.h,v 1.46 2005/10/14 15:17:02 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/xf86Pci.h,v 1.48 2007/02/13 18:30:10 tsi Exp $ */
 /*
  * Copyright 1998 by Concurrent Computer Corporation
  *
@@ -70,7 +70,7 @@
  *
  */
 /*
- * Copyright (c) 1999-2003 by The XFree86 Project, Inc.
+ * Copyright (c) 1999-2007 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -150,7 +150,9 @@
 #define PCI_CMD_STEPPING_ENABLE		0x0080
 #define PCI_CMD_SERR_ENABLE		0x0100
 #define PCI_CMD_BACKTOBACK_ENABLE	0x0200
+#define PCI_CMD_INTERRUPT_DISABLE	0x0400
 #define PCI_STAT_MASK			0xffff0000
+#define PCI_STAT_INTERRUPT_PENDING	0x00080000
 #define PCI_STAT_CAPABILITY		0x00100000
 #define PCI_STAT_66MHZ			0x00200000
 #define PCI_STAT_UDF			0x00400000
@@ -199,62 +201,117 @@
 
 /* Sub class values */
 /* 0x00 prehistoric subclasses */
-#define PCI_SUBCLASS_PREHISTORIC_MISC	0x00
-#define PCI_SUBCLASS_PREHISTORIC_VGA	0x01
+#define PCI_SUBCLASS_PREHISTORIC_MISC		0x00
+#define PCI_SUBCLASS_PREHISTORIC_VGA		0x01
 
 /* 0x01 mass storage subclasses */
 #define PCI_SUBCLASS_MASS_STORAGE_SCSI		0x00
 #define PCI_SUBCLASS_MASS_STORAGE_IDE		0x01
+#define PCI_IF_MASS_STORAGE_IDE_PRIMARY_MODE		0x01
+#define PCI_IF_MASS_STORAGE_IDE_PRIMARY_INDICATOR	0x02
+#define PCI_IF_MASS_STORAGE_IDE_SECONDARY_MODE		0x04
+#define PCI_IF_MASS_STORAGE_IDE_SECONDARY_INDICATOR	0x08
+#define PCI_IF_MASS_STORAGE_IDE_MASTER			0x80
 #define PCI_SUBCLASS_MASS_STORAGE_FLOPPY	0x02
 #define PCI_SUBCLASS_MASS_STORAGE_IPI		0x03
+#define PCI_SUBCLASS_MASS_STORAGE_RAID		0x04
+#define PCI_SUBCLASS_MASS_STORAGE_ATA		0x05
+#define PCI_IF_MASS_STORAGE_ATA_SINGLE_DMA		0x20
+#define PCI_IF_MASS_STORAGE_ATA_CHAINED_DMA		0x30
 #define PCI_SUBCLASS_MASS_STORAGE_MISC		0x80
 
 /* 0x02 network subclasses */
-#define PCI_SUBCLASS_NETWORK_ETHERNET	0x00
-#define PCI_SUBCLASS_NETWORK_TOKENRING	0x01
-#define PCI_SUBCLASS_NETWORK_FDDI	0x02
-#define PCI_SUBCLASS_NETWORK_MISC	0x80
+#define PCI_SUBCLASS_NETWORK_ETHERNET		0x00
+#define PCI_SUBCLASS_NETWORK_TOKENRING		0x01
+#define PCI_SUBCLASS_NETWORK_FDDI		0x02
+#define PCI_SUBCLASS_NETWORK_ATM		0x03
+#define PCI_SUBCLASS_NETWORK_ISDN		0x04
+#define PCI_SUBCLASS_NETWORK_WORLDFIP		0x05
+#define PCI_SUBCLASS_NETWORK_PICMIG		0x06
+#define PCI_SUBCLASS_NETWORK_MISC		0x80
 
 /* 0x03 display subclasses */
-#define PCI_SUBCLASS_DISPLAY_VGA	0x00
-#define PCI_SUBCLASS_DISPLAY_XGA	0x01
-#define PCI_SUBCLASS_DISPLAY_MISC	0x80
+#define PCI_SUBCLASS_DISPLAY_VGA		0x00
+#define PCI_IF_DISPLAY_VGA				0x00
+#define PCI_IF_DISPLAY_8514				0x01
+#define PCI_SUBCLASS_DISPLAY_XGA		0x01
+#define PCI_SUBCLASS_DISPLAY_3D			0x02
+#define PCI_SUBCLASS_DISPLAY_MISC		0x80
 
 /* 0x04 multimedia subclasses */
-#define PCI_SUBCLASS_MULTIMEDIA_VIDEO	0x00
-#define PCI_SUBCLASS_MULTIMEDIA_AUDIO	0x01
-#define PCI_SUBCLASS_MULTIMEDIA_MISC	0x80
+#define PCI_SUBCLASS_MULTIMEDIA_VIDEO		0x00
+#define PCI_SUBCLASS_MULTIMEDIA_AUDIO		0x01
+#define PCI_SUBCLASS_MULTIMEDIA_TELEPHONY	0x02
+#define PCI_SUBCLASS_MULTIMEDIA_MISC		0x80
 
 /* 0x05 memory subclasses */
-#define PCI_SUBCLASS_MEMORY_RAM		0x00
-#define PCI_SUBCLASS_MEMORY_FLASH	0x01
-#define PCI_SUBCLASS_MEMORY_MISC	0x80
+#define PCI_SUBCLASS_MEMORY_RAM			0x00
+#define PCI_SUBCLASS_MEMORY_FLASH		0x01
+#define PCI_SUBCLASS_MEMORY_MISC		0x80
 
 /* 0x06 bridge subclasses */
-#define PCI_SUBCLASS_BRIDGE_HOST	0x00
-#define PCI_SUBCLASS_BRIDGE_ISA		0x01
-#define PCI_SUBCLASS_BRIDGE_EISA	0x02
-#define PCI_SUBCLASS_BRIDGE_MC		0x03
-#define PCI_SUBCLASS_BRIDGE_PCI		0x04
-#define PCI_SUBCLASS_BRIDGE_PCMCIA	0x05
-#define PCI_SUBCLASS_BRIDGE_NUBUS	0x06
-#define PCI_SUBCLASS_BRIDGE_CARDBUS	0x07
-#define PCI_SUBCLASS_BRIDGE_RACEWAY	0x08
-#define PCI_SUBCLASS_BRIDGE_MISC	0x80
-#define PCI_IF_BRIDGE_PCI_SUBTRACTIVE	0x01
+#define PCI_SUBCLASS_BRIDGE_HOST		0x00
+#define PCI_SUBCLASS_BRIDGE_ISA			0x01
+#define PCI_SUBCLASS_BRIDGE_EISA		0x02
+#define PCI_SUBCLASS_BRIDGE_MC			0x03
+#define PCI_SUBCLASS_BRIDGE_PCI			0x04
+#define PCI_IF_BRIDGE_PCI_SUBTRACTIVE			0x01
+#define PCI_SUBCLASS_BRIDGE_PCMCIA		0x05
+#define PCI_SUBCLASS_BRIDGE_NUBUS		0x06
+#define PCI_SUBCLASS_BRIDGE_CARDBUS		0x07
+#define PCI_SUBCLASS_BRIDGE_RACEWAY		0x08
+#define PCI_IF_BRIDGE_RACEWAY_TRANSPARENT		0x00
+#define PCI_IF_BRIDGE_RACEWAY_ENDPOINT			0x01
+#define PCI_SUBCLASS_BRIDGE_PCI_SEMITRANSPARENT	0x09
+#define PCI_IF_BRIDGE_PCI_SEMITRANSPARENT_PRIMARY	0x40
+#define PCI_IF_BRIDGE_PCI_SEMITRANSPARENT_SECONDARY	0x80
+#define PCI_SUBCLASS_BRIDGE_INFINIBAND		0x0a
+#define PCI_SUBCLASS_BRIDGE_MISC		0x80
 
 /* 0x07 communications controller subclasses */
 #define PCI_SUBCLASS_COMMUNICATIONS_SERIAL	0x00
+#define PCI_IF_COMMUNICATIONS_SERIAL_XT			0x00
+#define PCI_IF_COMMUNICATIONS_SERIAL_16450		0x01
+#define PCI_IF_COMMUNICATIONS_SERIAL_16550		0x02
+#define PCI_IF_COMMUNICATIONS_SERIAL_16650		0x03
+#define PCI_IF_COMMUNICATIONS_SERIAL_16750		0x04
+#define PCI_IF_COMMUNICATIONS_SERIAL_16850		0x05
+#define PCI_IF_COMMUNICATIONS_SERIAL_16950		0x06
 #define PCI_SUBCLASS_COMMUNICATIONS_PARALLEL	0x01
+#define PCI_IF_COMMUNICATIONS_PARALLEL_GENERIC		0x00
+#define PCI_IF_COMMUNICATIONS_PARALLEL_BIDIRECTIONAL	0x01
+#define PCI_IF_COMMUNICATIONS_PARALLEL_ECP		0x02
+#define PCI_IF_COMMUNICATIONS_PARALLEL_IEEE1284		0x03
+#define PCI_IF_COMMUNICATIONS_PARALLEL_IEEE1284_TARGET	0xfe
 #define PCI_SUBCLASS_COMMUNICATIONS_MULTISERIAL	0x02
 #define PCI_SUBCLASS_COMMUNICATIONS_MODEM	0x03
+#define PCI_IF_COMMUNICATIONS_MODEM_GENERIC		0x00
+#define PCI_IF_COMMUNICATIONS_MODEM_16450		0x01
+#define PCI_IF_COMMUNICATIONS_MODEM_16550		0x02
+#define PCI_IF_COMMUNICATIONS_MODEM_16650		0x03
+#define PCI_IF_COMMUNICATIONS_MODEM_16750		0x04
+#define PCI_SUBCLASS_COMMUNICATIONS_GPIB	0x04
+#define PCI_SUBCLASS_COMMUNICATIONS_SMARTCARD	0x05
 #define PCI_SUBCLASS_COMMUNICATIONS_MISC	0x80
 
 /* 0x08 generic system peripherals subclasses */
 #define PCI_SUBCLASS_SYSPERIPH_PIC	0x00
+#define PCI_IF_SYSPERIPH_PIC_8259		0x00
+#define PCI_IF_SYSPERIPH_PIC_ISA		0x01
+#define PCI_IF_SYSPERIPH_PIC_EISA		0x02
+#define PCI_IF_SYSPERIPH_PIC_IO_APIC		0x10
+#define PCI_IF_SYSPERIPH_PIC_IOx_APIC		0x20
 #define PCI_SUBCLASS_SYSPERIPH_DMA	0x01
+#define PCI_IF_SYSPERIPH_DMA_8237		0x00
+#define PCI_IF_SYSPERIPH_DMA_ISA		0x01
+#define PCI_IF_SYSPERIPH_DMA_EISA		0x02
 #define PCI_SUBCLASS_SYSPERIPH_TIMER	0x02
+#define PCI_IF_SYSPERIPH_TIMER_8254		0x00
+#define PCI_IF_SYSPERIPH_TIMER_ISA		0x01
+#define PCI_IF_SYSPERIPH_TIMER_EISA		0x02
 #define PCI_SUBCLASS_SYSPERIPH_RTC	0x03
+#define PCI_IF_SYSPERIPH_RTC_GENERIC		0x00
+#define PCI_IF_SYSPERIPH_RTC_ISA		0x01
 #define PCI_SUBCLASS_SYSPERIPH_HOTPCI	0x04
 #define PCI_SUBCLASS_SYSPERIPH_MISC	0x80
 
@@ -264,6 +321,8 @@
 #define PCI_SUBCLASS_INPUT_MOUSE	0x02
 #define PCI_SUBCLASS_INPUT_SCANNER	0x03
 #define PCI_SUBCLASS_INPUT_GAMEPORT	0x04
+#define PCI_IF_INPUT_GAMEPORT_GENERIC		0x00
+#define PCI_IF_INPUT_GAMEPORT_EXTENDED		0x10
 #define PCI_SUBCLASS_INPUT_MISC		0x80
 
 /* 0x0a docking station subclasses */
@@ -281,16 +340,24 @@
 
 /* 0x0c serial bus controller subclasses */
 #define PCI_SUBCLASS_SERIAL_FIREWIRE		0x00
+#define PCI_IF_SERIAL_FIREWIRE_IEEE1394			0x00
+#define PCI_IF_SERIAL_FIREWIRE_IEEE1394_OPENHCI		0x10
 #define PCI_SUBCLASS_SERIAL_ACCESS		0x01
 #define PCI_SUBCLASS_SERIAL_SSA			0x02
 #define PCI_SUBCLASS_SERIAL_USB			0x03
+#define PCI_IF_SERIAL_USB_UHCS				0x00
+#define PCI_IF_SERIAL_USB_OHCS				0x10
+#define PCI_IF_SERIAL_USB_OTHER				0x80
+#define PCI_IF_SERIAL_USB_DEVICE			0xfe
 #define PCI_SUBCLASS_SERIAL_FIBRECHANNEL	0x04
 #define PCI_SUBCLASS_SERIAL_SMBUS		0x05
 
 /* 0x0d wireless controller subclasses */
 #define PCI_SUBCLASS_WIRELESS_IRDA		0x00
 #define PCI_SUBCLASS_WIRELESS_CONSUMER_IR	0x01
-#define PCI_SUBCLASS_WIRELESS_RF		0x02
+#define PCI_SUBCLASS_WIRELESS_RF		0x10
+#define PCI_SUBCLASS_WIRELESS_BLUETOOTH		0x11
+#define PCI_SUBCLASS_WIRELESS_BROADBAND		0x12
 #define PCI_SUBCLASS_WIRELESS_MISC		0x80
 
 /* 0x0e intelligent I/O controller subclasses */
@@ -309,6 +376,9 @@
 
 /* 0x11 data acquisition and signal processing controller subclasses */
 #define PCI_SUBCLASS_DATAACQ_DPIO	0x00
+#define PCI_SUBCLASS_DATAACQ_COUNTER	0x01
+#define PCI_SUBCLASS_DATAACQ_SYNCHRO	0x10
+#define PCI_SUBCLASS_DATAACQ_MANAGEMENT	0x20
 #define PCI_SUBCLASS_DATAACQ_MISC	0x80
 
 
@@ -386,6 +456,7 @@
 #define PCI_PCI_BRIDGE_IO_REG		0x1c
 #define PCI_PCI_BRIDGE_MEM_REG		0x20
 #define PCI_PCI_BRIDGE_PMEM_REG		0x24
+#define PCI_PCI_BRIDGE_ROM_REG		0x38
 
 #define PCI_PPB_IOBASE_EXTRACT(x)	(((x) << 8) & 0xFF00)
 #define PCI_PPB_IOLIMIT_EXTRACT(x)	(((x) << 0) & 0xFF00)
@@ -408,6 +479,7 @@
 #define PCI_CB_BRIDGE_CTL_PREFETCH_MEM1	0x200
 #define PCI_CB_BRIDGE_CTL_POST_WRITES	0x400
 
+#define PCI_CB_CAP_PTR			0x14
 #define PCI_CB_SEC_STATUS_REG		0x16	/* Secondary status */
 #define PCI_CB_PRIMARY_BUS_REG		0x18	/* PCI bus number */
 #define PCI_CB_CARD_BUS_REG		0x19	/* CardBus bus number */
@@ -653,10 +725,17 @@ typedef struct pci_cfg_regs {
 #endif
 		} b_u_io;
 	    } uio_rom;
-	    struct {
-		CARD32 rsvd3;		/* Offset 0x34 - 0x37 */
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+	    CARD8 capptr;		/* Offset 0x34 */
+	    CARD8 rsvd3[3];		/* Offset 0x35 - 0x37 */
+#else
+	    CARD8 rsvd3[3];		/* Offset 0x35 - 0x37 */
+	    CARD8 capptr;		/* Offset 0x34 */
+#endif
+	    union {
 		CARD32 rsvd4;		/* Offset 0x38 - 0x3b */
-	    } rsvd;
+		CARD32 br_rom;
+	    } ubr_rom;
 	} cd;
     } cx;
     union {				/* Offset 0x3c - 0x3f */
@@ -676,15 +755,15 @@ typedef struct pci_cfg_regs {
 #endif
 	    } mmii;
 	} mmii;
-	struct {				/* header type 1 */
+	struct {				/* header type 1 & 2 */
 #if X_BYTE_ORDER == X_BIG_ENDIAN
-	    CARD16 bridge_control;	/* upper 8 bits reserved */
-	    CARD8  rsvd2;
-	    CARD8  rsvd1;
+	    CARD16 bridge_control;
+	    CARD8  int_pin;
+	    CARD8  int_line;
 #else
-	    CARD8  rsvd1;
-	    CARD8  rsvd2;
-	    CARD16 bridge_control;	/* upper 8 bits reserved */
+	    CARD8  int_line;
+	    CARD8  int_pin;
+	    CARD16 bridge_control;
 #endif
 	} bctrl;
     } bm;
@@ -711,7 +790,7 @@ typedef struct pci_device {
     int	      funcnum;
     pciCfgSpc cfgspc;
     int	      basesize[7];	/* number of bits in base addr allocations */
-    Bool      minBasesize;
+    int       minBasesize;	/* was a Bool, now a bit mask */
     CARD32    listed_class;
     pointer   businfo;		/* pointer to secondary's bus info structure */
     Bool      fakeDevice;	/* Device added by system chipset support */
@@ -787,6 +866,8 @@ typedef enum {
 #define pci_prefetch_upper_mem_limit  cfgspc.regs.cx.cd.um_ssys_id.pftch_umem_limit
 #define pci_upper_io_base	      cfgspc.regs.cx.cd.uio_rom.b_u_io.io_ubase
 #define pci_upper_io_limit	      cfgspc.regs.cx.cd.uio_rom.b_u_io.io_ulimit
+#define pci_capptr		      cfgspc.regs.cx.cd.capptr
+#define pci_br_rom		      cfgspc.regs.cx.cd.ubr_rom.br_rom
 #define pci_int_line		      cfgspc.regs.bm.mmii.mmii.int_line
 #define pci_int_pin		      cfgspc.regs.bm.mmii.mmii.int_pin
 #define pci_min_gnt		      cfgspc.regs.bm.mmii.mmii.min_gnt
@@ -822,7 +903,8 @@ void	      pciSetBitsByte(PCITAG tag, int offset, CARD8 mask, CARD8 val);
 ADDRESS	      pciBusAddrToHostAddr(PCITAG tag, PciAddrType type, ADDRESS addr);
 ADDRESS	      pciHostAddrToBusAddr(PCITAG tag, PciAddrType type, ADDRESS addr);
 PCITAG	      pciTag(int busnum, int devnum, int funcnum);
-int	      pciGetBaseSize(PCITAG tag, int indx, Bool destructive, Bool *min);
+int	      pciGetBaseSize(pciConfigPtr device, int indx,
+			     Bool destructive, int *min);
 CARD32	      pciCheckForBrokenBase(PCITAG tag,int basereg);
 Bool          xf86LocatePciMemoryArea(PCITAG tag,
 				      char **devName, unsigned int *devOffset,
