@@ -25,7 +25,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/bitmap/Bitmap.c,v 1.7 2006/01/09 15:00:40 dawes Exp $ */
+/* $XFree86: xc/programs/bitmap/Bitmap.c,v 1.8 2007/04/09 15:37:19 tsi Exp $ */
 
 /*
  * Author:  Davor Matic, MIT X Consortium
@@ -1798,7 +1798,7 @@ SetValues(Widget old, Widget request, Widget new,
   }
   
   if (NE(bitmap.size)) {
-    Dimension width, height;
+    Dimension width = 0, height = 0;
     
     if (BWParseSize(newbw->bitmap.size, &width, &height)) { 
       ResizeGrid(newbw, width, height);
