@@ -1,4 +1,4 @@
-/* $Xorg: wire.c,v 1.4 2001/02/09 02:05:32 xorgcvs Exp $ */
+/* $XFree86: xc/programs/lbxproxy/di/wire.c,v 1.19 2006/06/19 13:43:26 tsi Exp $ */
 /*
 
 Copyright 1998  The Open Group
@@ -45,7 +45,6 @@ in this Software without prior written authorization from The Open Group.
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XFree86: xc/programs/lbxproxy/di/wire.c,v 1.17 2004/04/26 00:23:37 tsi Exp $ */
 
 #include "lbx.h"
 #include <stdio.h>
@@ -76,7 +75,7 @@ in this Software without prior written authorization from The Open Group.
 
 #include <X11/ICE/ICElib.h>
 #ifdef BIGREQS
-#include "bigreqstr.h"
+#include <X11/extensions/bigreqstr.h>
 #endif
 
 /*
@@ -1549,7 +1548,8 @@ ConnectToServer(dpy_name)
 	clients[0]->server = server;
 
 #ifdef NEED_UTSNAME
-	uname (&name);
+	if (uname(&name) < 0)
+	    name.nodename[0] = '\0';
 	(void) snprintf(my_host,sizeof(my_host),"%s",name.nodename);
 #else
         (void) gethostname (my_host,sizeof(my_host));
